; Assembly routines for DISLITE. TASM source
; mainly for taskswitching between decoded program & DISLITE main

; Routines:
;  void interrupt resume(void);
;     Resumes the DISLITE main program, from either a breakpoint or a
;     single step interrupt. Puts the current state in exestate, and
;     performs a longjmp() to myexestate.
      
;  void interrupt unbiosint(void);
;  void interrupt undiskint(void);
;  void interrupt undosint(void);
;     Resumes the DISLITE main program, from an unexpected interrupt.
;     puts the current state in exestate, and performs a longjmp().

;  void switchto(void);
;     Taskswitch to the program to decode.
;     Takes the state from exestate.

;  void terminate(void);
;     Terminate the current running program.
;     Used to terminate the program that is decoded.

; This file uses TASM IDEAL mode. Conversion to other formats is probably
; almost trivial.
   IDEAL
   MODEL COMPACT

; The STRUC Exe_State here *MUST* resemble the struct in the C file:
;   struct Exe_State {
;      void far* rSSSP;
;      void far* rCSIP;
;      struct Psp far* psp;
;      unsigned rAX, rBX, rCX, rDX, rSI, rDI, rBP, rDS, rES, rFLAGS;
;   };

STRUC Exe_State
  rSSSP  DD ?
  rCSIP  DD ?
  psp	 DD ?
  rAX	 DW ?
  rBX	 DW ?
  rCX	 DW ?
  rDX	 DW ?
  rSI	 DW ?
  rDI	 DW ?
  rBP	 DW ?
  rDS	 DW ?
  rES	 DW ?
  rFLAGS DW ?
ENDS

; declare the public available routines
PUBLIC _terminate
PUBLIC _switchto
PUBLIC _resume
PUBLIC _unbiosint
PUBLIC _undiskint
PUBLIC _undosint

; import some global variables from the C source
EXTRN _exestate:Exe_State
EXTRN _myexestate:WORD	; This (WORD) is a lie, but we don't need it more specific.
EXTRN _longjmp:NEAR

; Some magic constants
EXIT	    =	  4Ch
BIOSINT     =	  10h
DISKINT     =	  13h
DOSINT	    =	  21h

   CODESEG

; Terminate the current running program.
; Used to terminate the program that is decoded.
_terminate:
   mov ah,EXIT
   int DOSINT

; Taskswitch to the program to decode.
; Takes the state from exestate.
_switchto:
   mov ax,SEG _exestate
   mov ds,ax
   mov si,OFFSET _exestate
   mov ax,[WORD PTR ((Exe_State PTR si).rSSSP) + 2]
   cli
   mov ss,ax
   mov sp,[WORD PTR (Exe_State PTR si).rSSSP]
   sti
   push [(Exe_State PTR si).rFLAGS]
   push [WORD PTR ((Exe_State PTR si).rCSIP) + 2]
   push [WORD PTR (Exe_State PTR si).rCSIP]
   push [(Exe_State PTR si).rDS]
   mov es,[(Exe_State PTR si).rES]
   mov ax,[(Exe_State PTR si).rAX]
   mov bx,[(Exe_State PTR si).rBX]
   mov cx,[(Exe_State PTR si).rCX]
   mov dx,[(Exe_State PTR si).rDX]
   mov di,[(Exe_State PTR si).rDI]
   mov bp,[(Exe_State PTR si).rBP]
   mov si,[(Exe_State PTR si).rSI]
   pop ds
   iret

; Resumes the DISLITE main program, from either a breakpoint or a
; single step interrupt. Puts the current state in exestate, and
; performs a longjmp() to myexestate.
_resume:
   push ax
   mov al,1
   jmp _allresume

; Resumes the DISLITE main program, from an unexpected interrupt.
; puts the current state in exestate, and performs a longjmp().
_unbiosint:
   push ax
   mov al,BIOSINT
   jmp _allresume

_undiskint:
   push ax
   mov al,DISKINT
   jmp _allresume

_undosint:
   push ax
   mov al,DOSINT
;  jmp _allresume ; (fallthrough)

; The common part of the resume routines.
; AX contains the interrupt number that caused this. (1 = break or trap)
_allresume:
   push ds
   push si
   mov si,SEG _exestate
   mov ds,si
   mov si,OFFSET _exestate
   mov [(Exe_State PTR si).rBX],bx
   mov [(Exe_State PTR si).rCX],cx
   mov [(Exe_State PTR si).rDX],dx
   mov [(Exe_State PTR si).rDI],di
   mov [(Exe_State PTR si).rBP],bp
   mov [(Exe_State PTR si).rES],es
   pop [(Exe_State PTR si).rSI]
   pop [(Exe_State PTR si).rDS]
   pop [(Exe_State PTR si).rAX]
   pop [WORD PTR (Exe_State PTR si).rCSIP]
   pop [WORD PTR ((Exe_State PTR si).rCSIP) + 2]
   pop [(Exe_State PTR si).rFLAGS]
   mov [WORD PTR ((Exe_State PTR si).rSSSP) + 2],ss
   mov [WORD PTR (Exe_State PTR si).rSSSP],sp
; AX still contains the interrupt number
   cbw
   push ax
   mov ax,SEG _myexestate
   push ax
   mov ax,OFFSET _myexestate
   push ax
   call _longjmp  ; Never returns, but for argument passing's sake, we call.

END
